'use strict';

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  ref.insertAt;

  { return; }
}

styleInject();

var initialWebRTCStreamerOptions = {
    element: '',
    url: '',
};
var initialJanusVideoRoomOptions = {
    janusUrl: '',
    url: '',
    eventBus: undefined,
};

/**
 * Interface with WebRTC-streamer API
 */
var WebRTCStreamer = /** @class */ (function () {
    /**
     * Instantiate object
     * @constructor
     * @param args
     */
    function WebRTCStreamer(args) {
        if (args === void 0) { args = {}; }
        this.peerConnection = null;
        this.peerConnectionId = 0;
        this.iceServers = null;
        this.earlyCandidates = [];
        this.options = Object.assign({}, initialWebRTCStreamerOptions, args);
        if (!this.options.url) {
            this.options.url = "".concat(window.location.protocol, "//").concat(window.location.hostname, ":").concat(window.location.port);
        }
        this.mediaConstraints = { offerToReceiveAudio: true, offerToReceiveVideo: true };
        this.changeElement(this.options.element);
    }
    /**
     * Connect a WebRTC Stream to videoElement
     * @param videUrl
     * @param audioUrl
     * @param options
     * @param localStream
     * @param preferenceMime
     */
    WebRTCStreamer.prototype.connect = function (videUrl, audioUrl, options, localStream, preferenceMime) {
        var _this = this;
        this.disconnect();
        if (!this.iceServers) {
            fetch("".concat(this.options.url, "/api/getIceServers"))
                .then(this.handleHttpErrors)
                .then(function (res) {
                return res.json();
            })
                .then(function (res) {
                return _this.onReceiveGetIceServers(res, videUrl, audioUrl, options, localStream, preferenceMime);
            })
                .catch(function (error) { return _this.onError("getIceServers ".concat(error)); });
        }
        else {
            this.onReceiveGetIceServers(this.iceServers, videUrl, audioUrl, options, localStream, preferenceMime);
        }
    };
    /**
     * Disconnect a WebRTC Stream and clear videoElement source
     */
    WebRTCStreamer.prototype.disconnect = function () {
        var _this = this;
        var _a;
        if ((_a = this.element) === null || _a === void 0 ? void 0 : _a.srcObject) {
            // @ts-ignore
            this.element.srcObject.getTracks().forEach(function (track) {
                var _a, _b;
                track.stop();
                // @ts-ignore
                (_b = (_a = _this.element) === null || _a === void 0 ? void 0 : _a.srcObject) === null || _b === void 0 ? void 0 : _b.removeTrack(track);
            });
        }
        if (this.peerConnection) {
            fetch("".concat(this.options.url, "/api/hangup?peerid=").concat(this.peerConnectionId))
                .then(this.handleHttpErrors)
                .catch(function (error) { return _this.onError("hangup ".concat(error)); });
            try {
                this.peerConnection.close();
            }
            catch (e) {
                console.log("Failure close peer connection: ".concat(e));
            }
            this.peerConnection = null;
        }
    };
    WebRTCStreamer.prototype.changeElement = function (element) {
        if (typeof element === 'string') {
            var dom = document.querySelector(element);
            dom && (this.element = dom);
        }
        else {
            this.element = element;
        }
    };
    WebRTCStreamer.prototype.handleHttpErrors = function (res) {
        if (!res.ok) {
            throw Error(res.statusText);
        }
        return res;
    };
    /**
     * GetIceServers callback
     * @param iceServers
     * @param videoUrl
     * @param audioUrl
     * @param options
     * @param stream
     * @private
     */
    WebRTCStreamer.prototype.onReceiveGetIceServers = function (iceServers, videoUrl, audioUrl, options, stream, preferenceMime) {
        var _this = this;
        var _a;
        this.iceServers = iceServers;
        this.peerConnectionConfig = iceServers || { iceServers: [] };
        try {
            this.createPeerConnection();
            var callUrl_1 = "".concat(this.options.url, "/api/call?peerid=").concat(this.peerConnectionId, "&url=").concat(encodeURIComponent(videoUrl));
            if (audioUrl) {
                callUrl_1 += "&audiourl=".concat(encodeURIComponent(audioUrl));
            }
            if (options) {
                callUrl_1 += "&options=".concat(encodeURIComponent(options));
            }
            if (stream) {
                // @ts-ignore
                this.peerConnection.addStream(stream);
            }
            // clear early candidates
            this.earlyCandidates.length = 0;
            // create Offer
            (_a = this.peerConnection) === null || _a === void 0 ? void 0 : _a.createOffer(this.mediaConstraints).then(function (sessionDescription) {
                var _a, _b, _c, _d, _e, _f, _g;
                console.log("Create offer: ".concat(JSON.stringify(sessionDescription)));
                console.log("video codecs:".concat(Array.from(new Set((_b = (_a = RTCRtpReceiver.getCapabilities('video')) === null || _a === void 0 ? void 0 : _a.codecs) === null || _b === void 0 ? void 0 : _b.map(function (codec) { return codec.mimeType; })))));
                console.log("audio codecs:".concat(Array.from(new Set((_d = (_c = RTCRtpReceiver.getCapabilities('audio')) === null || _c === void 0 ? void 0 : _c.codecs) === null || _d === void 0 ? void 0 : _d.map(function (codec) { return codec.mimeType; })))));
                if (preferenceMime !== undefined) {
                    // set preference codec
                    var preferenceKind_1 = preferenceMime.split('/')[0];
                    var codecs = ((_e = RTCRtpReceiver.getCapabilities(preferenceKind_1)) === null || _e === void 0 ? void 0 : _e.codecs) || [];
                    var preferenceCodecs_1 = codecs.filter(function (codec) { return codec.mimeType === preferenceMime; });
                    console.log("preferenceCodecs: ".concat(JSON.stringify(preferenceCodecs_1)));
                    (_f = _this.peerConnection) === null || _f === void 0 ? void 0 : _f.getTransceivers().filter(function (transceiver) { return transceiver.receiver.track.kind === preferenceKind_1; }).forEach(function (item) {
                        if (item.setCodecPreferences !== undefined) {
                            item.setCodecPreferences(preferenceCodecs_1);
                        }
                    });
                }
                (_g = _this.peerConnection) === null || _g === void 0 ? void 0 : _g.setLocalDescription(sessionDescription).then(function () {
                    fetch(callUrl_1, {
                        method: 'POST',
                        body: JSON.stringify(sessionDescription),
                    })
                        .then(_this.handleHttpErrors)
                        .then(function (res) {
                        return res.json();
                    })
                        .then(function (res) {
                        _this.onReceiveCall(res);
                    })
                        .catch(function (error) { return _this.onError("call ".concat(error)); });
                }, function (error) {
                    console.log("setLocalDescription error: ".concat(JSON.stringify(error)));
                });
            }, function (error) {
                console.log("Create offer error: ".concat(JSON.stringify(error)));
            });
        }
        catch (e) {
            this.disconnect();
            console.log("connect error: ".concat(e));
        }
    };
    /**
     * AJAX callback for Error
     * @param status
     * @private
     */
    WebRTCStreamer.prototype.onError = function (status) {
        console.log("onError: ".concat(status));
    };
    /**
     * create RTCPeerConnection
     * @private
     */
    WebRTCStreamer.prototype.createPeerConnection = function () {
        var _this = this;
        console.log("createPeerConnection config: ".concat(JSON.stringify(this.peerConnectionConfig)));
        this.peerConnection = new RTCPeerConnection(this.peerConnectionConfig);
        this.peerConnectionId = Math.random();
        this.peerConnection.onicecandidate = function (event) {
            _this.onIceCandidate(event);
        };
        // @ts-ignore
        this.peerConnection.onaddstream = function (event) {
            _this.onAddStream(event);
        };
        this.peerConnection.oniceconnectionstatechange = function (evt) {
            var _a, _b;
            console.log("oniceconnectionstatechange state: ".concat((_a = _this.peerConnection) === null || _a === void 0 ? void 0 : _a.iceConnectionState));
            if (!_this.element) {
                return;
            }
            switch ((_b = _this.peerConnection) === null || _b === void 0 ? void 0 : _b.iceConnectionState) {
                case 'connected':
                    _this.element.style.opacity = '1.0';
                    break;
                case 'disconnected':
                    _this.element.style.opacity = '0.25';
                    break;
                case 'failed':
                case 'closed':
                    _this.element.style.opacity = '0.5';
                    break;
                case 'new':
                    _this.getIceCandidate();
                    break;
            }
        };
        this.peerConnection.ondatachannel = function (event) {
            console.log("remote datachannel created: ".concat(JSON.stringify(event)));
            event.channel.onopen = function () {
                console.log('remote datachannel open');
                this.send('remote channel opened');
            };
            event.channel.onmessage = function (event) {
                console.log("remote datachannel receive: ".concat(JSON.stringify(event.data)));
            };
        };
        // this.peerConnection.onicegatheringstatechange = () => {
        //   if (this.peerConnection?.iceGatheringState === 'complete') {
        //     const receivers = this.peerConnection.getReceivers()
        //
        //     receivers.forEach((receiver) => {
        //       if (receiver.track && receiver.track.kind === 'video') {
        //         console.log(`codecs: ${JSON.stringify(receiver.getParameters().codecs)}`)
        //       }
        //     })
        //   }
        // }
        try {
            var dataChannel = this.peerConnection.createDataChannel('ClientDataChannel');
            dataChannel.onopen = function () {
                console.log('local datachannel open');
                this.send('local channel opened');
            };
            dataChannel.onmessage = function (event) {
                console.log("local datachannel receiver: ".concat(JSON.stringify(event.data)));
            };
        }
        catch (e) {
            console.log("Cannot create datachannel error: ".concat(e));
        }
        console.log("Created RTCPeerConnection with config: ".concat(JSON.stringify(this.peerConnectionConfig)));
        return this.peerConnection;
    };
    /**
     * AJAX /call callback
     * @param dataJson
     * @private
     */
    WebRTCStreamer.prototype.onReceiveCall = function (dataJson) {
        var _this = this;
        var _a;
        console.log("offer: ".concat(JSON.stringify(dataJson)));
        var sessionDescription = new RTCSessionDescription(dataJson);
        (_a = this.peerConnection) === null || _a === void 0 ? void 0 : _a.setRemoteDescription(sessionDescription).then(function () {
            console.log('setRemoteDescription ok');
            while (_this.earlyCandidates.length) {
                var candidate = _this.earlyCandidates.shift();
                _this.addIceCandidate(_this.peerConnectionId, candidate);
            }
            _this.getIceCandidate();
        }, function (error) {
            console.log("setRemoteDescription error: ".concat(JSON.stringify(error)));
        });
    };
    WebRTCStreamer.prototype.addIceCandidate = function (id, candidate) {
        var _this = this;
        fetch("".concat(this.options.url, "/api/addIceCandidate?peerid=").concat(id), {
            method: 'POST',
            body: JSON.stringify(candidate),
        })
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            console.log("addIceCandidate ok: ".concat(res));
        })
            .catch(function (error) { return _this.onError("addIceCandidate ".concat(error)); });
    };
    WebRTCStreamer.prototype.getIceCandidate = function () {
        var _this = this;
        fetch("".concat(this.options.url, "/api/getIceCandidate?peerid=").concat(this.peerConnectionId))
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            _this.onReceiveCandidate(res);
        })
            .catch(function (error) { return _this.onError("getIceCandidate ".concat(error)); });
    };
    /**
     * AJAX /getIceCandidate callback
     * @param dataJson
     * @private
     */
    WebRTCStreamer.prototype.onReceiveCandidate = function (dataJson) {
        var _a, _b;
        console.log("candidate: ".concat(JSON.stringify(dataJson)));
        if (dataJson) {
            for (var i = 0; i < dataJson.length; i++) {
                var candidate = new RTCIceCandidate(dataJson[i]);
                console.log("Adding ICE candidate: ".concat(JSON.stringify(candidate)));
                (_a = this.peerConnection) === null || _a === void 0 ? void 0 : _a.addIceCandidate(candidate).then(function () {
                    console.log('addIceCandidate OK');
                }, function (error) {
                    console.log("addIceCandidate error: ".concat(JSON.stringify(error)));
                });
            }
            (_b = this.peerConnection) === null || _b === void 0 ? void 0 : _b.addIceCandidate();
        }
    };
    /**
     * RTCPeerConnection AddTrack callback
     * @param event
     * @private
     */
    WebRTCStreamer.prototype.onAddStream = function (event) {
        var _this = this;
        console.log("Remote track added: ".concat(JSON.stringify(event)));
        this.element.srcObject = event.stream;
        var promise = this.element.play();
        if (promise !== undefined) {
            promise.catch(function (error) {
                var _a;
                console.warn("error: ".concat(error));
                (_a = _this.element) === null || _a === void 0 ? void 0 : _a.setAttribute('controls', 'true');
            });
        }
    };
    /**
     * RTCPeerConnection IceCandidate callback
     * @param event
     * @private
     */
    WebRTCStreamer.prototype.onIceCandidate = function (event) {
        var _a;
        if (event.candidate) {
            if ((_a = this.peerConnection) === null || _a === void 0 ? void 0 : _a.currentRemoteDescription) {
                this.addIceCandidate(this.peerConnectionId, event.candidate);
            }
            else {
                this.earlyCandidates.push(event.candidate);
            }
        }
        else {
            console.log('End of candidates.');
        }
    };
    return WebRTCStreamer;
}());

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

/// <reference types="google.maps" />
var HtmlMapMarker = /** @class */ (function (_super) {
    __extends(HtmlMapMarker, _super);
    /**
     * Instantiate object
     * accepts latLng, html, map etc
     * @constructor
     * @param args
     */
    function HtmlMapMarker(args) {
        var _this = _super.call(this) || this;
        _this.latLng = args.latLng;
        _this.html = args.html;
        _this.map = args.map;
        _this.width = args.width;
        _this.height = args.height;
        _this.setMap(args.map);
        return _this;
    }
    /**
     * create a div to hold the marker content
     */
    HtmlMapMarker.prototype.createDiv = function () {
        var _this = this;
        this.div = document.createElement('div');
        this.div.style.position = 'absolute';
        if (this.width && this.height) {
            this.div.style.width = "".concat(this.width, "px");
            this.div.style.height = "".concat(this.height, "px");
        }
        if (this.html) {
            this.div.innerHTML = this.html;
        }
        google.maps.event.addDomListener(this.div, 'click', function (event) {
            event.stopPropagation();
            google.maps.event.trigger(_this, 'click', event);
        });
    };
    /**
     * position the marker div
     */
    HtmlMapMarker.prototype.positionDiv = function () {
        var point = this.getProjection().fromLatLngToDivPixel(this.latLng);
        if (point) {
            var left = point.x - this.div.clientWidth / 2;
            this.div.style.left = "".concat(left, "px");
            var top_1 = point.y - this.div.clientHeight / 2;
            this.div.style.top = "".concat(top_1, "px");
        }
    };
    /**
     * add the marker div to the map
     */
    HtmlMapMarker.prototype.onAdd = function () {
        if (!this.div) {
            this.createDiv();
            var panes = this.getPanes();
            panes === null || panes === void 0 ? void 0 : panes.overlayMouseTarget.appendChild(this.div);
        }
        this.positionDiv();
    };
    /**
     * reposition the marker div
     */
    HtmlMapMarker.prototype.draw = function () {
        this.positionDiv();
    };
    /**
     * remove the marker div from the map
     */
    HtmlMapMarker.prototype.onRemove = function () {
        var _a;
        if (this.div) {
            (_a = this.div.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(this.div);
            this.div = undefined;
        }
    };
    /**
     * get the marker position
     */
    HtmlMapMarker.prototype.getPosition = function () {
        return this.latLng;
    };
    /**
     * remove the marker from the map
     */
    HtmlMapMarker.prototype.detach = function () {
        if (this.getMap()) {
            this.setMap(null);
        }
    };
    /**
     * attach the marker to the map
     * @param map
     */
    HtmlMapMarker.prototype.attach = function (map) {
        if (!this.getMap()) {
            this.setMap(map || this.map);
        }
    };
    /**
     * set size of the marker div
     * @param width
     * @param height
     */
    HtmlMapMarker.prototype.setSize = function (width, height) {
        this.width = width;
        this.height = height;
        if (this.div) {
            this.div.style.width = "".concat(this.width, "px");
            this.div.style.height = "".concat(this.height, "px");
        }
    };
    return HtmlMapMarker;
}(google.maps.OverlayView));

/**
 * Interface with Janus Gateway Video Room and WebRTC-streamer API
 */
var JanusVideoRoom = /** @class */ (function () {
    /**
     * Instantiate object
     * @constructor
     * @param args
     */
    function JanusVideoRoom(args) {
        if (args === void 0) { args = {}; }
        this.connection = {};
        this.options = Object.assign({}, initialJanusVideoRoomOptions, args);
        this.options.url =
            this.options.url ||
                "".concat(window.location.protocol, "//").concat(window.location.hostname, ":").concat(window.location.port);
    }
    /**
     * Ask to publish a stream from WebRTC-streamer in a Janus Video Room user
     * @param janusRoomId - id of the Janus Video Room to join
     * @param url - WebRTC stream to publish
     * @param name - name in Janus Video Room
     */
    JanusVideoRoom.prototype.join = function (janusRoomId, url, name) {
        var _this = this;
        var createRequest = {
            janus: 'create',
            transaction: Math.random().toString(),
        };
        fetch(this.options.janusUrl, {
            method: 'POST',
            body: JSON.stringify(createRequest),
        })
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            _this.onCreateSession(res, janusRoomId, url, name);
        })
            .catch(function (error) { return _this.onError("create ".concat(error)); });
    };
    JanusVideoRoom.prototype.leave = function (janusRoomId, url, name) {
        var _this = this;
        var connection = this.connection["".concat(janusRoomId, "_").concat(url, "_").concat(name)];
        if (connection) {
            var sessionId = connection.sessionId;
            var pluginId = connection.pluginId;
            var leaveRequest = {
                janus: 'message',
                body: {
                    request: 'unpublish',
                },
                transaction: Math.random().toString(),
            };
            fetch("".concat(this.options.janusUrl, "/").concat(sessionId, "/").concat(pluginId), {
                method: 'POST',
                body: JSON.stringify(leaveRequest),
            })
                .then(this.handleHttpErrors)
                .then(function (res) {
                return res.json();
            })
                .then(function (res) {
                console.log("leave janus room answer: ".concat(res));
            })
                .catch(function (error) { return _this.onError("leave ".concat(error)); });
        }
    };
    JanusVideoRoom.prototype.handleHttpErrors = function (res) {
        if (!res.ok) {
            throw Error(res.statusText);
        }
        return res;
    };
    JanusVideoRoom.prototype.emit = function (name, state) {
        var _a;
        (_a = this.options.eventBus) === null || _a === void 0 ? void 0 : _a.emit('state', name, state);
    };
    /**
     * Janus callback for Session Creation
     * @param dataJson
     * @param janusRoomId
     * @param url
     * @param name
     * @private
     */
    JanusVideoRoom.prototype.onCreateSession = function (dataJson, janusRoomId, url, name) {
        var _this = this;
        var sessionId = dataJson.data.id;
        console.log("onCreateSession sessionId: ".concat(sessionId));
        // attach to video room plugin
        var attachRequest = {
            janus: 'attach',
            plugin: 'janus.plugin.videoroom',
            transaction: Math.random().toString(),
        };
        fetch("".concat(this.options.janusUrl, "/").concat(sessionId), {
            method: 'POST',
            body: JSON.stringify(attachRequest),
        })
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            _this.onPluginsAttached(res, janusRoomId, url, name, sessionId);
        })
            .catch(function (error) { return _this.onError("attach ".concat(error)); });
    };
    /**
     * Janus callback for Video Room Plugins Connection
     * @param dataJson
     * @param janusRoomId
     * @param url
     * @param name
     * @param sessionId
     * @private
     */
    JanusVideoRoom.prototype.onPluginsAttached = function (dataJson, janusRoomId, url, name, sessionId) {
        var _this = this;
        var pluginId = dataJson.data.id;
        console.log("onPluginsAttached pluginId: ".concat(pluginId));
        this.emit(name, 'joining');
        var joinRequest = {
            janus: 'message',
            body: {
                request: 'join',
                room: janusRoomId,
                ptype: 'publisher',
                display: name,
            },
            transaction: Math.random().toString(),
        };
        fetch("".concat(this.options.janusUrl, "/").concat(sessionId, "/").concat(pluginId), {
            method: 'POST',
            body: JSON.stringify(joinRequest),
        })
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (response) {
            _this.onJoinRoom(response, janusRoomId, url, name, sessionId, pluginId);
        })
            .catch(function (error) { return _this.onError("join ".concat(error)); });
    };
    /**
     * Janus callback for Video Room Joined
     * @param dataJson
     * @param janusRoomId
     * @param url
     * @param name
     * @param sessionId
     * @param pluginId
     * @private
     */
    JanusVideoRoom.prototype.onJoinRoom = function (dataJson, janusRoomId, url, name, sessionId, pluginId) {
        var _this = this;
        console.log("onJoinRoom: ".concat(JSON.stringify(dataJson)));
        fetch("".concat(this.options.janusUrl, "/").concat(sessionId, "?rid=").concat(new Date().getTime(), "&maxev=1"))
            .then(this.handleHttpErrors)
            .then(function (response) {
            return response.json();
        })
            .then(function (response) {
            _this.onJoinRoomResult(response, janusRoomId, url, name, sessionId, pluginId);
        })
            .catch(function (error) { return _this.onError("join answer ".concat(error)); });
    };
    /**
     * Janus callback for Video Room Joined
     * @param dataJson
     * @param janusRoomId
     * @param url
     * @param name
     * @param sessionId
     * @param pluginId
     * @private
     */
    JanusVideoRoom.prototype.onJoinRoomResult = function (dataJson, janusRoomId, url, name, sessionId, pluginId) {
        var _this = this;
        console.log("onJoinRoomResult: ".concat(JSON.stringify(dataJson)));
        if (dataJson.plugindata.data.videoroom === 'joined') {
            // register connection
            this.connection["".concat(janusRoomId, "_").concat(url, "_").concat(name)] = { sessionId: sessionId, pluginId: pluginId };
            // member of the room
            var publishers = dataJson.plugindata.data.publishers;
            for (var i = 0; i < publishers.length; i++) {
                var publisher = publishers[i];
                this.emit(publisher.display, 'up');
            }
            if (name) {
                // notify new state
                this.emit(name, 'joined');
                var peerId_1 = Math.random().toString();
                var createOfferUrl = void 0;
                if (typeof url === 'string') {
                    createOfferUrl = "".concat(this.options.url, "/api/createOffer?peerid=").concat(peerId_1, "&url=").concat(encodeURIComponent(url));
                }
                else {
                    createOfferUrl = "".concat(this.options.url, "/api/createOffer?peerid=").concat(peerId_1, "&url=").concat(encodeURIComponent(url.video || ''));
                    if (url.audio) {
                        createOfferUrl += "&audiourl=".concat(encodeURIComponent(url.audio));
                    }
                    if (url.options) {
                        createOfferUrl += "&options=".concat(encodeURIComponent(url.options));
                    }
                }
                fetch(createOfferUrl)
                    .then(this.handleHttpErrors)
                    .then(function (res) {
                    return res.json();
                })
                    .then(function (res) {
                    _this.onCreateOffer(res, name, sessionId, pluginId, peerId_1);
                })
                    .catch(function (error) { return _this.onError("createOffer ".concat(error)); });
            }
            else {
                // start long polling
                this.longPoll(null, name, sessionId);
            }
        }
        else {
            this.emit(name, 'joining room failed');
        }
    };
    /**
     * WebRTC streamer callback for Offer
     * @param dataJson
     * @param name
     * @param sessionId
     * @param pluginId
     * @param peerId
     * @private
     */
    JanusVideoRoom.prototype.onCreateOffer = function (dataJson, name, sessionId, pluginId, peerId) {
        var _this = this;
        console.log("onCreateOffer: ".concat(JSON.stringify(dataJson)));
        this.emit(name, 'publishing');
        var publishReq = {
            janus: 'message',
            body: { request: 'publish', video: true, audio: true, data: true },
            jsep: dataJson,
            transaction: Math.random().toString(),
        };
        fetch("".concat(this.options.janusUrl, "/").concat(sessionId, "/").concat(pluginId), {
            method: 'POST',
            body: JSON.stringify(publishReq),
        })
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            _this.onPublishStream(res, name, sessionId, pluginId, peerId);
        })
            .catch(function (error) { return _this.onError("publish ".concat(error)); });
    };
    /**
     * Janus callback for WebRTC stream is published
     * @param dataJson
     * @param name
     * @param sessionId
     * @param pluginId
     * @param peerId
     * @private
     */
    JanusVideoRoom.prototype.onPublishStream = function (dataJson, name, sessionId, pluginId, peerId) {
        var _this = this;
        console.log("onPublishStream: ".concat(JSON.stringify(dataJson)));
        fetch("".concat(this.options.janusUrl, "/").concat(sessionId, "?rid=").concat(new Date().getTime(), "&maxev=1"))
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            _this.onPublishStreamResult(res, name, sessionId, pluginId, peerId);
        })
            .catch(function (error) { return _this.onError("publish answer ".concat(error)); });
    };
    /**
     * Janus callback for WebRTC stream is published
     * @param dataJson
     * @param name
     * @param sessionId
     * @param pluginId
     * @param peerId
     * @private
     */
    JanusVideoRoom.prototype.onPublishStreamResult = function (dataJson, name, sessionId, pluginId, peerId) {
        var _this = this;
        console.log("onPublishStreamResult: ".concat(JSON.stringify(dataJson)));
        if (dataJson.jsep) {
            fetch("".concat(this.options.url, "/api/setAnswer?peerid=").concat(peerId), {
                method: 'POST',
                body: JSON.stringify(dataJson.jsep),
            })
                .then(this.handleHttpErrors)
                .then(function (res) {
                return res.json();
            })
                .then(function (response) {
                _this.onSetAnswer(response, name, sessionId, pluginId, peerId);
            })
                .catch(function (error) { return _this.onError("setAnswer ".concat(error)); });
        }
        else {
            this.emit(name, 'publishing failed (no SDP)');
        }
    };
    /**
     * WebRTC streamer callback for Answer
     * @param dataJson
     * @param name
     * @param sessionId
     * @param pluginId
     * @param peerId
     * @private
     */
    JanusVideoRoom.prototype.onSetAnswer = function (dataJson, name, sessionId, pluginId, peerId) {
        var _this = this;
        console.log("onSetAnswer: ".concat(JSON.stringify(dataJson)));
        fetch("".concat(this.options.url, "/api/getIceCandidate?peerid=").concat(peerId))
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            _this.onReceiveCandidate(res, name, sessionId, pluginId);
        })
            .catch(function (error) { return _this.onError("getIceCandidate ".concat(error)); });
    };
    /**
     * WebRTC streamer callback for ICE candidate
     * @param dataJson
     * @param name
     * @param sessionId
     * @param pluginId
     */
    JanusVideoRoom.prototype.onReceiveCandidate = function (dataJson, name, sessionId, pluginId) {
        var _this = this;
        console.log("onReceiveCandidate answer: ".concat(JSON.stringify(dataJson)));
        for (var i = 0; i < dataJson.length; i++) {
            // send ICE candidate to Janus
            var candidateRequest = {
                janus: 'trickle',
                candidate: dataJson[i],
                transaction: Math.random().toString(),
            };
            fetch("".concat(this.options.janusUrl, "/").concat(sessionId, "/").concat(pluginId), {
                method: 'POST',
                body: JSON.stringify(candidateRequest),
            })
                .then(this.handleHttpErrors)
                .then(function (res) {
                return res.json();
            })
                .then(function (res) {
                console.log("onReceiveCandidate janus answer: ".concat(JSON.stringify(res)));
            })
                .catch(function (error) { return _this.onError("setAnswer ".concat(error)); });
        }
        // start long polling
        this.longPoll(null, name, sessionId);
    };
    /**
     * Janus callback for Long Polling
     * @param dataJson
     * @param name
     * @param sessionId
     * @private
     */
    JanusVideoRoom.prototype.longPoll = function (dataJson, name, sessionId) {
        var _this = this;
        if (dataJson) {
            console.log("poll evt: ".concat(JSON.stringify(dataJson)));
            if (dataJson.janus === 'webrtcup') {
                // notify connection
                this.emit(name, 'up');
                // start keep alive
                setInterval(function () {
                    _this.keepAlive(sessionId);
                }, 10000);
            }
            else if (dataJson.janus === 'hangup') {
                // notify connection
                this.emit(name, 'down');
            }
            else if (dataJson.janus === 'event') {
                // member of the room
                var publishers = dataJson.plugindata.data.publishers;
                if (publishers) {
                    for (var i = 0; i < publishers.length; i++) {
                        var publisher = publishers[i];
                        this.emit(publisher.display, 'up');
                    }
                }
            }
        }
        fetch("".concat(this.options.janusUrl, "/").concat(sessionId, "?rid=").concat(new Date().getTime(), "&maxev=1"))
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            _this.longPoll(res, name, sessionId);
        })
            .catch(function (error) { return _this.onError("long poll answer ".concat(error)); });
    };
    /**
     * Janus callback for keepAlive Session
     * @param sessionId
     * @private
     */
    JanusVideoRoom.prototype.keepAlive = function (sessionId) {
        var _this = this;
        var keepAliveReq = {
            janus: 'keepalive',
            session_id: sessionId,
            transaction: Math.random().toString(),
        };
        fetch("".concat(this.options.janusUrl, "/").concat(sessionId), {
            method: 'POST',
            body: JSON.stringify(keepAliveReq),
        })
            .then(this.handleHttpErrors)
            .then(function (res) {
            return res.json();
        })
            .then(function (res) {
            console.log("keepAlive answer: ".concat(JSON.stringify(res)));
        })
            .catch(function (error) { return _this.onError("keepAlive ".concat(error)); });
    };
    /**
     * Janus callback for Error
     * @param status
     * @private
     */
    JanusVideoRoom.prototype.onError = function (status) {
        console.log("onError: ".concat(status));
    };
    return JanusVideoRoom;
}());

exports.HtmlMapMarker = HtmlMapMarker;
exports.JanusVideoRoom = JanusVideoRoom;
exports.WebRTCStreamer = WebRTCStreamer;
//# sourceMappingURL=index.cjs.js.map
